// MyMainWindow.cpp: Implementierung der Klasse MyMainWindow.
//
//////////////////////////////////////////////////////////////////////

#include "MyMainWindow.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////


MyMainWindow::MyMainWindow( QWidget* parent, const char* name, WFlags f )
	: QMainWindow( parent, name, f )
{
	// Konstuktor
	setCaption("Vokabel Trainer");
	setIcon( NULL );

	central	 = new QFrame( this );

	menubar	 = new QHBoxLayout ( -1, "ButtonLayout" );
	vokabar	 = new QHBoxLayout ( -1, "EditLayout" );
	voklabel = new QVBoxLayout ( -1, "EditLabel" );
	vokedit	 = new QVBoxLayout ( -1, "EditLines" );
	editbar	 = new QHBoxLayout ( -1, "EditButtonLayout2" );
	layout	 = new QVBoxLayout( central, 0, -1, "MainLayout" );

	benter	 = new QPushButton( central );
	blearn	 = new QPushButton( central );
	bload	 = new QPushButton( central );
	bnew	 = new QPushButton( central );
	boptions = new QPushButton( central );
	bhelp	 = new QPushButton( central );

	lang1	 = new QLabel( line1, TEXT_LANGUAGE_1 ":", central );
	lang2	 = new QLabel( line2, TEXT_LANGUAGE_2 ":", central );
	statusl	 = new QLabel( NULL,  TEXT_STATUS     ":", central );

	line1	 = new QLineEdit( central );
	line2	 = new QLineEdit( central );
	statust	 = new QLineEdit( central );
	line1->setMaxLength( EDIT_MAXLENGTH );
	line2->setMaxLength( EDIT_MAXLENGTH );
	statust->setDisabled(true);
	statust->setText( PROGRAM_NAME " Version " PROGRAM_VERSION );

	setButtonDefaults( benter,	BUTTON_ENTER );
	setButtonDefaults( blearn,	BUTTON_LEARN );
	setButtonDefaults( bload,	BUTTON_LOAD );
	setButtonDefaults( bnew,	BUTTON_NEW );
	setButtonDefaults( boptions,BUTTON_OPTIONS );
	setButtonDefaults( bhelp,	BUTTON_HELP );
	
	menubar->addWidget( benter );
	menubar->addWidget( blearn );
	menubar->addWidget( bload );
	menubar->addWidget( bnew );
	menubar->addWidget( boptions );
	menubar->addWidget( bhelp );

	voklabel->addWidget( lang1 );
	voklabel->addWidget( lang2 );
	voklabel->addWidget( statusl );

	vokedit->addWidget( line1 );
	vokedit->addWidget( line2 );
	vokedit->addWidget( statust );

	vokabar->addLayout( voklabel );
	vokabar->addLayout( vokedit );

	layout->addLayout( menubar );
	layout->addLayout( vokabar );

	poptions = new QPopupMenu( this );
	CHECK_PTR ( poptions );
	poptions->insertItem( POPUP_STATISTIC,	this, SLOT(showStatistics()) );
	poptions->insertItem( POPUP_SHOWALL,	this, SLOT(showAll()) );
	mcheckCase=
	poptions->insertItem( POPUP_CASESENSI,	this, SLOT(toggleCase()) );
	poptions->insertItem( POPUP_RESETSTAT,	this, SLOT(resetLearnStatus()) );
	poptions->insertItem( POPUP_LANGNAME,	this, SLOT(setLanguage()) );
	poptions->insertItem( POPUP_REVERSE,	this, SLOT(convertLanguage()) );
	poptions->setFont( QFont( FONT_NAME, FONT_SIZE ));
	poptions->setItemChecked(mcheckCase, true);

	this->connect(blearn, SIGNAL(clicked()), this, SLOT(learnMode()));
	this->connect(benter, SIGNAL(clicked()), this, SLOT(enterMode()));
	this->connect(bload,  SIGNAL(clicked()), this, SLOT(load()));
	this->connect(bnew,   SIGNAL(clicked()), this, SLOT(save()));
	this->connect(bhelp,  SIGNAL(clicked()), this, SLOT(infoHelp()));
	this->connect(line1,  SIGNAL(returnPressed()), this, SLOT(getlines()));
	this->connect(line2,  SIGNAL(returnPressed()), this, SLOT(getlines()));

	boptions->setPopup( poptions );

	this->setFont( QFont( FONT_NAME, FONT_SIZE ));
	this->setCentralWidget( central );

	helpWindow = new MyHelpWindow( central, PROGRAM_SHORT ": " TEXT_HELP );
	helpWindow->setSource( getCWD() + "/" + HELP_FILENAME );
	showWindow = new MyShowWindow( central, PROGRAM_SHORT ": " TEXT_SHOWALL );

	this->setMinimumSize( 600, 150 );
	vok2edit = "";
	dataOK(false);
};

MyMainWindow::~MyMainWindow()
{
	// Destruktor bleibt leer, weil er nur zum Programmende aufgerufen wird
	// und das raeumt dann eh das Laufzeitsystem weg
};


void MyMainWindow::setButtonDefaults( QPushButton *button, const QString &name )
{
	// Standardeinstellungen fuer die Knoepfe festlegen

	button->setText( name );
	button->setFlat( false );
	button->setMinimumSize( 100, 30 );
};


void MyMainWindow::enterMode()
{
	// in den Eingabemodus wechseln
	
	blearn->setOn( false );
	benter->setOn( true );
	line1->setDisabled( false );
	line1->setFocus();
	vok2edit=line1->text();
	line2->setText(vokData->search(vok2edit));
	vokData->deleteLearnList();
	setStatus("");
};

void MyMainWindow::learnMode()
{
	// in den Lernmodus wechseln

	benter->setOn( false );
	blearn->setOn( true );
	line1->setDisabled( true );
	line1->setText("");
	line2->setFocus();
	line2->setText("");
	if(vokData->createLearnList())
	{
		enterMode();
		allLearned();
	}
	else
	{
		learn(false);
	}
};

void MyMainWindow::dataOK(bool ok)
{
	// wenn Daten vorliegen alle Eingabemoeglichkeiten freigeben
	benter->setOn( false );
	benter->setToggleButton( ok );
	benter->setDisabled( !ok );

	blearn->setOn( false );
	blearn->setToggleButton( ok );
	blearn->setDisabled( !ok );
	
	boptions->setDisabled( !ok );

	line1->setDisabled( !ok );
	line1->setText("");
	
	line2->setDisabled( !ok );
	line2->setText("");
};


void MyMainWindow::allLearned()
{
	// alle Vokabeln wurden erfolgreich gelernt,
	// bzw. es liegen keine ungelernten vor

	line1->setText("");
	line2->setText("");
	enterMode();
	setStatus("Keine ungelernten Vokabeln");
}


void MyMainWindow::learn(bool check)
{
	// im Lernmodus arbeiten:
	// check zeigt an, ob die Eingabe ueberprueft werden soll.

	QString entered;
	QString shouldbe;
	MyVokEntry *entry;
	bool correct;

	if(check)
	{
		entry	= vokData->getLearnEntry(0);
		if(entry == NULL)
		{
			allLearned();
			return;
		}
		entered = line2->text();
		shouldbe= entry->getTranslated();
		if(poptions->isItemChecked( mcheckCase ))
		{
			correct = (entered.compare(shouldbe) == 0);
		}
		else
		{
			correct = (entered.lower().compare(shouldbe.lower()) == 0);
		}
		vokData->setLearnResult( correct );
		if( correct )
		{
			setStatus( TEXT_RIGHT "!", TEXT_GO_ON );
		}
		else
		{
			setStatus( TEXT_WRONG "!", QString( TEXT_RIGHT ": ")+entry->getTranslated() );
		}

		autosave();
	}

	entry = vokData->getLearnEntry(getrand());
	if(entry == NULL)
	{
		enterMode();
		allLearned();
	}
	else
	{
		line1->setText(entry->getOriginal());
		line2->setText("");
	}
}


void MyMainWindow::getlines()
{
	// Eingabezeilen verarbeiten;

	QString line1data = line1->text();
	QString line2data = line2->text();

	if(blearn->isOn())
	{
		learn(true);
	}
	else
	{
		if( line1data.isEmpty() )
		{
			// line1 leer
			line1->setFocus();
		}
		else
		{
			// line1 nicht leer
			if( line2data.isEmpty() )
			{
				// line2 leer
				vok2edit = line1data;
				line2->setText(vokData->search(line1data));
			}
		}

		if( line1data.isEmpty() ) return;

		if( line2->hasFocus() )
		{
			if( line2data.isEmpty() ) return;

			// Daten nur uebernehmen, wenn line2 den Fokus hat
			if( !vok2edit.isEmpty() )
			{
				// soll Datensatz geaendert werden?
				vokData->insertEntry( vok2edit, "-" ); // loeschen
			}
			vokData->insertEntry( line1data, line2data );
				
			line1->setText("");
			line2->setText("");
			line1->setFocus();
			setStatus( QString( TEXT_ENTERED ) + ": " + line1data );
			vok2edit = "";
		}
		else
		{
			line2->setFocus();
		}
	}
	autosave();
};


void MyMainWindow::load()
{
	// Daten Laden

	QString filename;
	if(vokData)
	{
		delete vokData;
		vokData = NULL;
	}
	dataOK(false);
	filename = QFileDialog::getOpenFileName(NULL, DATAFILE_FILTER);
	if( filename.isEmpty() )
	{
		return;
	}
	realLoad(filename);
}


void MyMainWindow::loadAtStart(char *filename)
{
	// Daten beim Start laden (wird von main.cpp aufgerufen)

	vokData = new MyVokData();
	realLoad(QString(filename));
}


void MyMainWindow::realLoad(QString filename)
{
	// eigentliche Datenladeroutine

	vokData = new MyVokData();
	vokData->setFilename(filename);
	if(!vokData->load())
	{
		setStatus( TEXT_LOADFAILED );
		dataOK(false);
		return;
	}
	setLangLabels();
	dataOK(true);
	learnMode();
}


void MyMainWindow::autosave()
{
	// eigentliches Speichern der Daten, wird
	// nach Aenderungen an den Daten automatisch aufgerufen

	if(!vokData->save())
	{
		setStatus( TEXT_SAVEFAILED );
	}
}

void MyMainWindow::save()
{
	// neue Vokabeldatei anlegen

	QString filename;
	QFile file;

	dataOK(false);
	line1->setText("");
	line2->setText("");

	if(vokData)
	{
		delete vokData;
		vokData = NULL;
	}
	filename = QFileDialog::getSaveFileName(NULL, DATAFILE_FILTER);
	file.setName(filename);
	if(file.exists())
	{
		if( fileExistsWarn() )
		{
			return;			
		}
	}
	if(filename.isEmpty()) return;
	vokData = new MyVokData();
	vokData->setFilename(filename);
	dataOK(true);
	setLanguage();
	enterMode();
}


bool MyMainWindow::fileExistsWarn()
{
	// Warnmeldung, die ausgegeben wird, falls die zu erzeugende Datei
	// schon existiert.
	// true als Rueckgabewert signalisiert: es soll ueberschrieben werden

	return (QMessageBox::warning( this, TEXT_CREATE_NEW, TEXT_FILE_EXIST,
			QMessageBox::Ok, QMessageBox::Abort ) != QMessageBox::Ok) ;
}


void MyMainWindow::setLangLabels()
{
	// die Textlabel vor den Eingabezeilen setzen
	
	lang1->setText( vokData->language1 + ":" );
	lang2->setText( vokData->language2 + ":" );
}


void MyMainWindow::showStatistics()
{
	// kurze Statistikausgabe

	MyVokEntry *entry;
	QString qstring,qnumVok, qsumKnown, qsumNotKnown, qavStat;
	int numVok=0, sumKnown=0, sumNotKnown=0, sumLStatus=0;


	vokData->setToFirstEntry();
	while( (entry = vokData->getNextEntry()) )
	{
		numVok++;
		sumKnown+=entry->getKnown();
		sumNotKnown+=entry->getNotKnown();
		sumLStatus+=entry->getStatus();
	}

	if(numVok == 0)
	{
		qstring = TEXT_NO_VOK;
	}
	else
	{
		qnumVok.setNum(numVok);
		qsumKnown.setNum(sumKnown);
		qsumNotKnown.setNum(sumNotKnown);
		qavStat.setNum((double)sumLStatus/numVok);

		qstring = TEXT_LEARNSTATE ":"
			TEXT_STAT_1 ": "+qnumVok+
			TEXT_STAT_2 ": "+qsumKnown+
			TEXT_STAT_3 ": "+qsumNotKnown+
			TEXT_STAT_4 ": "+qavStat;
	}

	QMessageBox::information( this, TEXT_LEARNSTATE, qstring );
}

void MyMainWindow::resetLearnStatus()
{
	// alle Vokabeln auf einen bestimmten Lernstatus setzen

	MyVokEntry *entry;
	bool ok;
	int value;

	value = QInputDialog::getInteger( TEXT_ENTER_NUML, TEXT_ENTER_NUML, LSTATUS_DEFAULT,
										1, 9, 1, &ok, this );

	vokData->setToFirstEntry();
	while( (entry = vokData->getNextEntry()) )
	{
		entry->setStatus( value );
	}
	autosave();
}


void MyMainWindow::setLanguage()
{
	// Namen der Sprachen abfragen und setzen

	QString qstring;
	bool ok;
	qstring = QInputDialog::getText( TEXT_ENTER_LNG1, TEXT_ENTER_LNG1, QLineEdit::Normal,
									 vokData->language1, &ok, this );
	if(ok) vokData->language1 = qstring;
	qstring = QInputDialog::getText( TEXT_ENTER_LNG2, TEXT_ENTER_LNG2, QLineEdit::Normal,
									 vokData->language2, &ok, this );
	if(ok) vokData->language2 = qstring;
	setLangLabels();
	autosave();
}


void MyMainWindow::convertLanguage()
{
	// Sprachen 1 und 2 vertauscht abspeichern

	QString filename;
	QFile file;
	MyVokEntry *entry;
	MyVokData *conVokData;

	filename = QFileDialog::getSaveFileName(NULL, DATAFILE_FILTER);
	file.setName(filename);
	if(file.exists())
	{
		if( fileExistsWarn() )
		{
			return;			
		}
	}
	if( filename.isEmpty() ) return;

	conVokData = new MyVokData();

	conVokData->setFilename(filename);
	conVokData->language1 = vokData->language2;
	conVokData->language2 = vokData->language1;

	vokData->setToFirstEntry();
	while( (entry = vokData->getNextEntry()) )
	{
		conVokData->insertEntry( entry->getTranslated(), entry->getOriginal() );
	}
	if( !conVokData->save() )
	{
		setStatus( TEXT_SAVEFAILED );
	}
	delete conVokData;
}


void MyMainWindow::infoHelp()
{
	// das Hilfefenster ein- oder ausblenden.

	helpWindow->toggle();
}


void MyMainWindow::showAll()
{
	// alle Vokabeln anzeigen

	vokData->setToFirstEntry();
	if( !vokData->getNextEntry() )
	{
		QMessageBox::information( this, TEXT_SHOWALL, TEXT_NO_VOK );
		return;
	}

	showWindow->populate( vokData );
	if(!showWindow->getSelected().isEmpty())
	{
		line1->setText(showWindow->getSelected());
		enterMode();
	}
	showWindow->depopulate(); // to be sure!
}


void MyMainWindow::setStatus(QString qstring1, QString qstring2)
{
	// Text der Statuszeile setzen
	// 1 Parameter: rechts aendern, links "Status:"
	// 2 Parameter: erst links, dann rechts

	if( qstring2.isEmpty() )
	{
		statusl->setText( TEXT_STATUS ":" );
		statust->setText( qstring1 );
	}
	else
	{
		statusl->setText( qstring1 );
		statust->setText( qstring2 );
	}
}

void MyMainWindow::toggleCase()
{
	// Ueberpruefung Gross-/Kleinschreibung aendern

	poptions->setItemChecked( mcheckCase, !poptions->isItemChecked( mcheckCase ) );
}
