// MyShowWindow.cpp: Implementierung der Klasse MyShowWindow.
//
//////////////////////////////////////////////////////////////////////

#include "MyShowWindow.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////


MyShowWindow::MyShowWindow( QWidget *parent, const char *name )
	: QDialog( parent, name, TRUE,
		WStyle_Customize | WStyle_NormalBorder |
		WStyle_Maximize | WStyle_Title | WStyle_SysMenu )
{
	// Konstruktor

	firstItem	= NULL;
	lastItem	= NULL;
	layout		= new QVBoxLayout( this );
	table		= new QTable( this, "VokTable" );
	close		= new QPushButton( this );
	close->setText( "Schliessen" );
	layout->addWidget( table );
	layout->addWidget( close );
	this->connect( this, SIGNAL(destroyed()), this, SLOT(catchClose()) );
	this->connect( close, SIGNAL(clicked()), this, SLOT(catchClose()) );
	this->connect( table, SIGNAL(currentChanged(int,int)), this, SLOT(catchClick(int,int)) );
	this->resize( SHOW_WINDOW_X_SIZE, SHOW_WINDOW_Y_SIZE );
	close->setFont( QFont(FONT_NAME, FONT_SIZE) );
}


MyShowWindow::~MyShowWindow()
{
	// wird auch nur bei Programmende aufgerufen
}


void MyShowWindow::addItem( QTableItem *qitem )
{
	// ein weiteres Element in QTable einfuegen
	// und fuer spaeteres loeschen merken

	MyTableItemList *item;

	item = new MyTableItemList( qitem );
	if(lastItem)
	{
		lastItem->next = item;
	}
	else
	{
		firstItem = item;
	}
	lastItem = item;
}


void MyShowWindow::populate( MyVokData *vokData )
{
	// Tabelle fuellen und anzeigen

	QString			qstring;
	MyVokEntry		*entry;
	QTableItem		*qitem;
	int				row=0;

	selected = "";
	table->setNumCols(3);
	table->setLeftMargin( 0 );
	table->verticalHeader()->hide();
	table->setSelectionMode(QTable::NoSelection);

	for( vokData->setToFirstEntry(); (entry = vokData->getNextEntry()); row++ )
	{
		table->setNumRows(row+1);

		qstring.setNum(entry->getStatus());
		qitem = new QTableItem(table , QTableItem::Never, qstring);
		addItem(qitem);
		table->setItem( row, 0, qitem );

		qitem = new QTableItem(table , QTableItem::Never, entry->getOriginal());
		addItem(qitem);
		table->setItem( row, 1, qitem );

		qitem = new QTableItem(table , QTableItem::Never, entry->getTranslated());
		addItem(qitem);
		table->setItem( row, 2, qitem );
	}

	table->horizontalHeader()->setLabel( 0, TABLE_HEADER_1 );
	table->horizontalHeader()->resizeSection( 0, SHOW_COLUMN_1 );
	table->horizontalHeader()->setLabel( 1, vokData->language1 );
	table->horizontalHeader()->resizeSection( 1, SHOW_COLUMN_2 );
	table->horizontalHeader()->setLabel( 2, vokData->language2 );
	table->horizontalHeader()->resizeSection( 2, SHOW_COLUMN_3 );

	table->removeSelection(table->currentSelection());

	this->exec();
}


void MyShowWindow::depopulate( int getitem )
{
	// Tabelle verstecken und loeschen

	this->hide();
	int item = 0;
	
	while(firstItem)
	{
		if(getitem == item++) 
		{
			selected = firstItem->entry->text();
		}
		lastItem = firstItem;
		firstItem = firstItem->next;
		delete lastItem->entry;
		delete lastItem;
	}

	firstItem	= NULL;
	lastItem	= NULL;
}


void MyShowWindow::catchClick(int row, int col /*, int button, const QPoint & mousePos*/)
{
	// Click in die Tabelle abfangen und Zeile merken

	if(!col) return;
	table->setCurrentCell( row, 0 );
	depopulate( row*3 + 1 );
}

void MyShowWindow::catchClose()
{
	// Schliessen abfangen

	depopulate();
}

QString	MyShowWindow::getSelected()
{
	// Vokabel bei der geklickt wurde abfragen

	return selected;
}
