// MyVokData.cpp: Implementierung der Klasse MyVokData.
//
//////////////////////////////////////////////////////////////////////

#include "MyVokData.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////


MyVokData::MyVokData()
{
	// Konstruktor

	fulllist	= NULL;
	deleteFullList(); // set fulllist;
	learnlist	= NULL;
	filename	= "";
	language1	= TEXT_LANGUAGE_1;
	language2	= TEXT_LANGUAGE_2;
	lstatusdefault = LSTATUS_DEFAULT;
}


MyVokData::~MyVokData()
{
	// Destruktor (wird wirklich mal aufgerufen)

	deleteLearnList();
	deleteFullList();
	delete fulllist;
}


QString MyVokData::search(QString& original)
{
	// eine Uebersetzung suchen

	QString translated("");
	MyVokList *elem;

	for( elem = fulllist->next; ; elem = elem->next )
	{
		if (!elem) break;
		if (QString::compare(original, elem->entry->getOriginal()) == 0) break;
	}
	
	if( elem )
	{
		translated = elem->entry->getTranslated();
	}
	
	return translated;
}


bool MyVokData::setFilename(QString filename_)
{
	// Dateinamen setzen und evtl. Endung anhaengen!

	filename = filename_;
	if( QString::compare(
		filename.right(QString(DATAFILE_EXT).length()).lower(),
		QString(DATAFILE_EXT).lower() ) ) filename += DATAFILE_EXT;
	return true;
}


bool MyVokData::load()
{
	// Datei laden

	QFile qfile(filename);
	QString qstring;

	deleteFullList();
	fulllist = new MyVokList(NULL,NULL);

	if(!qfile.open( IO_ReadOnly )) return false;
	
	QTextStream qstream( &qfile );

	if(qstream.eof()) return false;
	qstring = qstream.readLine();

	if(qstring.compare( PROGRAM_SHORT )) return false;

	if(qstream.eof()) return false;
	language1 = qstream.readLine();

	if(qstream.eof()) return false;
	language2 = qstream.readLine();

	/* Now read the data */
	while( !qstream.eof() )
	{
		MyVokEntry *entry = new MyVokEntry(qstream.readLine());
		if(QString::compare(entry->getOriginal(),QString("")))
		{
			insertEntry(entry);
		}
	}

	qfile.close();
	return true;
}


bool MyVokData::save()
{
	// Daten speichern

	QFile qfile(filename);
	MyVokList *list;
	QString qstring;

	if(!qfile.open( IO_WriteOnly | IO_Truncate )) return false;

	QTextStream qstream( &qfile );
	qstream.setEncoding( QTextStream::Unicode );

	qstream << PROGRAM_SHORT << endl;
	qstream << language1     << endl;
	qstream << language2     << endl;

	for(list = fulllist->next; list; list = list->next)
	{
		qstream << list->entry->write() << endl; 
	}

	qfile.close();
	return true;
}

bool MyVokData::createLearnList()
{
	// Liste aller noch zu lernenden Vokabeln erzeugen

	MyVokList *fhelp, *lhelp;
	deleteLearnList();

	lhelp = NULL;
	for(fhelp = fulllist->next; fhelp; fhelp = fhelp->next)
	{
		if( fhelp->entry->getStatus() )
		{
			if(lhelp)
			{
				lhelp->next = new MyVokList(fhelp->entry);
				lhelp = lhelp->next;
			}
			else
			{
				learnlist = new MyVokList(fhelp->entry);
				lhelp = learnlist;
			}
		}
	}
	learnp = learnlist;
	llearnp = learnlist;
	return learnlist == NULL;
}


void MyVokData::deleteLearnList()
{
	// Liste aller noch zu lernenden Vokabeln loeschen

	MyVokList *help;

	while(learnlist)
	{
		help = learnlist;
		learnlist = learnlist->next;
		delete help;
	}
	learnlist = NULL;
	learnp	  = NULL;
	llearnp	  = NULL;
}



MyVokEntry* MyVokData::getLearnEntry(unsigned int offset)
{
	// neue Vokabel aus Lernliste holen
	// 0 = aktuelle

	if(learnlist == NULL) return NULL;

	while(offset)
	{
		llearnp = learnp;
		if(learnp->next)
		{
			learnp=learnp->next;
		}
		else
		{
			learnp=learnlist;
		}
		offset--;
	}
	return learnp->entry;
}


bool MyVokData::setLearnResult(bool correct)
{
	// in Datensatz schreiben, ob Eingabe richtig war

	learnp->entry->setStatus( correct ? MyVokEntry::statusKnown :
										MyVokEntry::statusNotKnown );
	if( !learnp->entry->getStatus() )
	{
		createLearnList();
	}
	return learnlist == NULL;
}


void MyVokData::deleteFullList()
{
	MyVokList *help;

	while(fulllist)
	{
		help = fulllist;
		fulllist = fulllist->next;
		if(help->entry) delete help->entry;
		delete help;
	}
	fulllist = new MyVokList(NULL, NULL);
}


void MyVokData::insertEntry( QString original, QString translated )
{
	// Datensatz in Liste einfuegen (Wrapper fuer "Klartext")

	MyVokEntry *entry = new MyVokEntry( original, translated );
	entry->setStatus( lstatusdefault );
	insertEntry( entry );
}



void MyVokData::insertEntry( MyVokEntry *entry )
{
	// Datensatz in Liste einfuegen, diesmal in Echt und Lowlevel
	// die meisten der Speicherschutzverletzungen wurden hier begangen!

	MyVokList *prevle, *thisle, *nextle;

	for( prevle = fulllist; ; prevle = thisle )
	{
		thisle = prevle->next;
		if(!thisle) break;
		if(QString::compare(entry->getOriginal(), thisle->entry->getOriginal()) <= 0)
			break;
	}

	if(thisle)
	{
		// Listeintrag existiert
		nextle = thisle->next;

		if( !thisle->entry->getOriginal().compare( entry->getOriginal() ) )
		{
			// Eintraege gleich, Listeintrag muss geloescht werden;
			if(thisle->entry) delete thisle->entry;
			delete thisle;
			prevle->next = nextle;
		}
	}

	if( !entry->getTranslated().compare( DELETE_MARK ) )
	{
		// Loeschkeinzeichen, Eintrag weg
		delete entry;
	}
	else
	{
		// Kein Loeschen, sondern einfuegen
		thisle = new MyVokList(entry, prevle->next);
		prevle->next = thisle;
	}
}


void MyVokData::setToFirstEntry()
{
	// Zeiger auf ersten Eintrag setzen

	fulllistp = fulllist->next;
}


MyVokEntry *MyVokData::getNextEntry()
{
	// Eintrag holen und Zeiger einen weiter schieben
	// wird NULL zurueckgegeben, dann ist der Zeiger hinter dem
	// letztem Element

	MyVokList *retval;

	retval = fulllistp;
	if(fulllistp)
	{
		fulllistp = fulllistp->next;
		return retval->entry;
	}
	return NULL;
}
